#!/usr/bin/env python3

# Copyright (C) 2022, Jan-Philipp Litza (PLUTEX) <jpl@plutex.de>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# https://nlnetlabs.nl/projects/unbound/about/

# changes by thl-cmk[at]outlook[dot]com
# 2024-04-21: removed Union -> no need "int | float" should do
#             added levels_upper_NOERROR to default parameters -> show up in info line
# 2024-05-22: changed time for get_rate from section to system time
#             added output in case get_rate is initialising (unknown state in discovery)
#             removed default levels for unbound_answers
#             added params for unwanted replies


from time import time as now_time
from typing import (
    Mapping,
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    GetRateError,
    Result,
    Service,
    State,
    check_levels,
    get_rate,
    get_value_store,
    register,
)

from cmk.base.plugins.agent_based.utils.unbound import (
    UnboundSection,
    render_qps,
)


def discover_unbound_answers(section: UnboundSection) -> DiscoveryResult:
    if 'num.answer.rcode.SERVFAIL' in section:
        yield Service()


def check_unbound_answers(params: Mapping, section: UnboundSection) -> CheckResult:
    key_prefix = 'num.answer.rcode.'
    now = now_time()

    total = sum(
        value for key, value in section.items()
        if key.startswith(key_prefix)
    )

    init_counters = False
    for key, value in section.items():
        if not key.startswith(key_prefix):
            continue
        answer = key[len(key_prefix):]

        try:
            rate = get_rate(
                get_value_store(),
                f'unbound_answers_{answer}',
                now,
                value,
                raise_overflow=True,
            )
        except GetRateError as e:
            if not init_counters:
                yield Result(state=State.OK, summary=str(e))
                init_counters = True
        else:
            levels_upper = params.get(f'levels_upper_{answer}')
            if levels_upper is not None and len(levels_upper) == 3:
                # levels on the ratio of answers
                levels_upper = (
                    levels_upper[0] * total,
                    levels_upper[1] * total,
                )
            yield from check_levels(
                value=rate,
                levels_upper=levels_upper,
                metric_name=f'unbound_answers_{answer}',
                render_func=render_qps,
                label=answer,
                notice_only=f'levels_upper_{answer}' not in params,
            )


register.check_plugin(
    name='unbound_answers',
    service_name='Unbound Answers',
    sections=['unbound'],
    discovery_function=discover_unbound_answers,
    check_function=check_unbound_answers,
    check_default_parameters={
        # 'levels_upper_NOERROR': (101, 101),
        # 'levels_upper_SERVFAIL': (10, 100),
        # 'levels_upper_REFUSED': (10, 100),
    },
    check_ruleset_name='unbound_answers',
)
