#!/usr/bin/env python3

# Copyright (C) 2022, Jan-Philipp Litza (PLUTEX) <jpl@plutex.de>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# https://nlnetlabs.nl/projects/unbound/about/

# changes by thl-cmk[at]outlook[dot]com
# 2024-04-21: removed Union -> no need "int | float" should do
#             added levels_upper_NOERROR to default parameters -> show up in info line
# 2024-05-22: changed time for get_rate from section to system time
#             added output in case get_rate is initialising (unknown state in discovery)
#             removed default levels for unbound_answers
#             added params for unwanted replies


from time import time as now_time
from typing import (
    Any,
    Mapping,
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Service,
    check_levels,
    get_rate,
    get_value_store,
    register,
    render,
)

from cmk.base.plugins.agent_based.utils.unbound import (
    UnboundSection,
    render_qps,
)


def discover_unbound_cache(section: UnboundSection) -> DiscoveryResult:
    if 'total.num.cachehits' in section and 'total.num.cachemiss' in section:
        yield Service()


def check_unbound_cache(
        params: Mapping[str, Any],
        section: UnboundSection,
) -> CheckResult:
    cumulative_cache_hits = section.get('total.num.cachehits')
    cumulative_cache_miss = section.get('total.num.cachemiss')
    now = now_time()

    if None in (cumulative_cache_hits, cumulative_cache_miss, now):
        return

    cache_hits = get_rate(
        get_value_store(),
        'unbound_cache_hits',
        now,
        cumulative_cache_hits,
        raise_overflow=True,
    )
    cache_miss = get_rate(
        get_value_store(),
        'unbound_cache_miss',
        now,
        cumulative_cache_miss,
        raise_overflow=True,
    )
    total = cache_hits + cache_miss
    hit_perc = (cache_hits / float(total)) * 100.0 if total != 0 else 100.0

    yield from check_levels(
        value=cache_miss,
        metric_name='cache_misses_rate',
        levels_upper=params.get('cache_misses'),
        render_func=render_qps,
        label='Cache Misses',
        notice_only=True,
    )

    yield from check_levels(
        value=cache_hits,
        metric_name='cache_hit_rate',
        render_func=render_qps,
        label='Cache Hits',
        notice_only=True,
    )

    yield from check_levels(
        value=hit_perc,
        metric_name='cache_hit_ratio',
        levels_lower=params.get('cache_hits'),
        render_func=render.percent,
        label='Cache Hit Ratio',
    )


register.check_plugin(
    name='unbound_cache',
    service_name='Unbound Cache',
    sections=['unbound'],
    discovery_function=discover_unbound_cache,
    check_function=check_unbound_cache,
    check_default_parameters={},
    check_ruleset_name='unbound_cache',
)

